package nl.v2b.lalaland.application;

import nl.v2b.lalaland.application.exceptions.CategoryNotFoundException;
import nl.v2b.lalaland.data.SpringCategoryRepository;
import nl.v2b.lalaland.domain.Category;
import nl.v2b.lalaland.presentation.dtos.CategoryDTO;
import org.springframework.stereotype.Service;

import jakarta.transaction.Transactional;
import java.util.List;

@Service("categoryService")
@Transactional
public class CategoryService {

	private final SpringCategoryRepository categoryRepository;

	public CategoryService(SpringCategoryRepository categoryRepository) {
		this.categoryRepository = categoryRepository;
	}

	public List<Category> getAllCategories() { return categoryRepository.findAll(); }
	public Category getCategoryByID(int id) { return categoryRepository.findById(id).orElseThrow(CategoryNotFoundException::new); }

	public Category createCategory(CategoryDTO dto) {
		Category category = new Category(dto.getTitle());
		return categoryRepository.save(category);
	}

	public Category updateCategory(int id,CategoryDTO dto){
		Category category = categoryRepository.findById(id).orElseThrow(CategoryNotFoundException::new);
		category.setTitle(dto.getTitle());

		return categoryRepository.save(category);
	}

	public void deleteCategory(int id){
		this.categoryRepository.deleteById(id);
	}
}
