package nl.v2b.lalaland.application;

import nl.v2b.lalaland.application.exceptions.ChallengeAlreadyPassedException;
import nl.v2b.lalaland.application.exceptions.ChallengeNotFoundException;
import nl.v2b.lalaland.data.SpringChallengeRepository;
import nl.v2b.lalaland.data.SpringTextSubmissionRepository;
import nl.v2b.lalaland.domain.Player;
import nl.v2b.lalaland.domain.State;
import nl.v2b.lalaland.domain.challenges.TextChallenge;
import nl.v2b.lalaland.domain.submission.TextSubmission;
import nl.v2b.lalaland.domain.User;

import org.springframework.stereotype.Service;

import jakarta.transaction.Transactional;
import java.util.List;

@Service("textSubmissionService")
@Transactional
public class TextSubmissionService {
	private final SpringTextSubmissionRepository textSubmissionRepository;
	private final SpringChallengeRepository challengeRepository;

	public TextSubmissionService(SpringTextSubmissionRepository textSubmissionRepository, SpringChallengeRepository challengeRepository) {
		this.textSubmissionRepository = textSubmissionRepository;
		this.challengeRepository = challengeRepository;
	}

	public List<TextSubmission> getAllTextSubmissionsByChallengeId(int id) {
		return textSubmissionRepository.findAllByChallenge_Id(id);
	}

	public List<TextSubmission> getAllTextSubmissionsByUserAndChallenge(int challengeId, User user) {
		return textSubmissionRepository.findAllByChallenge_IdAndPlayer_Id(challengeId, user.getId());
	}

	public TextSubmission doTextSubmission(int challengeId, User user, String attempt) {
		if (!textSubmissionRepository.findAllByChallenge_IdAndPlayer_Id(challengeId, user.getId()).isEmpty()) {
			List<TextSubmission> submissions = textSubmissionRepository.findAllByChallenge_IdAndPlayer_Id(challengeId, user.getId());
			for (TextSubmission submission : submissions) {
				if (submission.getState() == State.PASSED)
					throw new ChallengeAlreadyPassedException("You've already passed this challenge");
			}
		}
		TextChallenge challenge = (TextChallenge) challengeRepository.findById(challengeId).orElseThrow(ChallengeNotFoundException::new);
		TextSubmission gradedSubmission = challenge.autoGradeTextSubmission(new TextSubmission(attempt, challenge, (Player) user));
		challenge.addSubmission(gradedSubmission);
		return textSubmissionRepository.save(gradedSubmission);
	}
}
