package nl.v2b.lalaland.domain.challenges;

import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import nl.v2b.lalaland.data.AnswerConverter;
import nl.v2b.lalaland.domain.Category;
import nl.v2b.lalaland.domain.State;
import nl.v2b.lalaland.domain.submission.TextSubmission;
import nl.v2b.lalaland.storage.domain.LalaFile;

import jakarta.persistence.*;
import java.util.List;
import java.util.stream.Collectors;

@Entity
@Data
@EqualsAndHashCode(callSuper = false)
@NoArgsConstructor
public class TextChallenge extends Challenge {
	@Convert(converter = AnswerConverter.class)
	private List<String> correctAnswers;

	public TextChallenge(int pointValue,
                         String description,
                         String title,
                         List<String> correctAnswers, LalaFile file, Category category) {
		super(pointValue, description, title, file, category);
		setCorrectAnswers(correctAnswers);
	}

	public void setCorrectAnswers(List<String> correctAnswers) {
		this.correctAnswers = correctAnswers.stream().map(String::toLowerCase).collect(Collectors.toList()); //always lowercase answers.
	}
	public TextSubmission autoGradeTextSubmission(TextSubmission submission) {
		submission.setState(correctAnswers.contains(submission.getAnswer().toLowerCase())?State.PASSED:State.FAILED);
		return submission;
	}
	public ChallengeType getType() {
		return ChallengeType.Text; //future: manually grade option.
	}
}
