package nl.v2b.lalaland.presentation;

import nl.v2b.lalaland.application.CategoryService;
import nl.v2b.lalaland.presentation.dtos.CategoryDTO;
import nl.v2b.lalaland.presentation.dtos.CategoryResponseDTO;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.security.RolesAllowed;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/category")
public class CategoryController {

	private final CategoryService service;

	public CategoryController(CategoryService service) {
		this.service = service;
	}
	@GetMapping("/all")
	public List<CategoryResponseDTO> getAllCategories() {
		return service.getAllCategories().stream().map(CategoryResponseDTO::new).collect(Collectors.toList());
	}

	@GetMapping("/{id}")
	public CategoryResponseDTO getCategory(@PathVariable int id) {
		return new CategoryResponseDTO(service.getCategoryByID(id));
	}
	@RolesAllowed("ADMIN")
	@PostMapping("/create")
	public CategoryResponseDTO createCategory(@RequestBody CategoryDTO categoryDTO) {
		return new CategoryResponseDTO(service.createCategory(categoryDTO));
	}
	@RolesAllowed("ADMIN")
	@PatchMapping("/update/{id}")
	public CategoryResponseDTO updateCategory(@PathVariable int id, @RequestBody CategoryDTO categoryDTO) {
		return new CategoryResponseDTO(service.updateCategory(id,categoryDTO));
	}
	@RolesAllowed("ADMIN")
	@DeleteMapping("/delete/{id}")
	public void deleteCategory(@PathVariable int id) {
		service.deleteCategory(id);
	}

}
