package nl.v2b.lalaland.presentation;
import nl.v2b.lalaland.application.ChallengeService;
import nl.v2b.lalaland.domain.challenges.Challenge;
import nl.v2b.lalaland.domain.challenges.MediaChallenge;
import nl.v2b.lalaland.domain.challenges.TextChallenge;
import nl.v2b.lalaland.presentation.dtos.ChallengeArchiveDTO;
import nl.v2b.lalaland.presentation.dtos.ChallengeDTO;
import nl.v2b.lalaland.presentation.dtos.ChallengeResponseDto;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.security.RolesAllowed;
import java.util.List;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/api/challenge")
public class ChallengeController {
	private final ChallengeService service;

	public ChallengeController(ChallengeService service){
		this.service = service;
	}
	@RolesAllowed("JUDGE")
	@GetMapping("/all")
	public List<ChallengeResponseDto> getAllChallenges(){
		return service.getAllChallenges().stream().map(ChallengeResponseDto::new).collect(Collectors.toList());
	}
	@GetMapping("/all/user")
	public List<ChallengeResponseDto> getAllGeldigChallenges(){
		return service.getAllChallenges().stream().
				filter(challenge -> !challenge.isArchive()).
				map(ChallengeResponseDto::new).collect(Collectors.toList());
	}
	@GetMapping("/media")
	public List<ChallengeResponseDto> getAllMediaChallenges(){
		return service.getAllChallenges().stream().
				filter(challenge -> challenge.getClass() == MediaChallenge.class)
				.map(ChallengeResponseDto::new).collect(Collectors.toList());
	}

	@GetMapping("/{id}")
	public ChallengeResponseDto getChallenge(@PathVariable int id){
		return new ChallengeResponseDto(service.getChallenge(id));
	}

	@RolesAllowed("JUDGE")
	@GetMapping("/{id}/answers")
	public List<String> getCorrectAnswers(@PathVariable int id) {
		return ((TextChallenge) service.getChallenge(id)).getCorrectAnswers();
	}

	@RolesAllowed("ADMIN")
	@PostMapping("/create")
	public ChallengeResponseDto createChallenge(@RequestBody ChallengeDTO challengeDTO){
		return new ChallengeResponseDto(service.createNewChallenge(challengeDTO));
	}

	@RolesAllowed("ADMIN")
	@PatchMapping("/update/{id}")
	public Challenge updateChallenge(@RequestBody ChallengeDTO challengeDTO,  @PathVariable int id){
		return service.updateChallenge(challengeDTO, id);
	}

	@RolesAllowed("ADMIN")
	@DeleteMapping("/{id}")
	public void deleteChallenge(@PathVariable int id){
		service.deleteChallenge(id);
	}

	@RolesAllowed("ADMIN")
	@PatchMapping("/archive/{id}")
	public ChallengeResponseDto archiveChallenge(@PathVariable int id, @RequestBody ChallengeArchiveDTO archiveDTO){
		return new ChallengeResponseDto(service.archiveChallenge(id,archiveDTO.isArchive()));
	}
}
