package nl.v2b.lalaland.presentation;

import lombok.Setter;
import nl.v2b.lalaland.application.ImageSubmissionService;
import nl.v2b.lalaland.domain.User;
import nl.v2b.lalaland.domain.submission.ImageSubmission;
import nl.v2b.lalaland.presentation.dtos.SubmissionDTO;
import nl.v2b.lalaland.presentation.dtos.MediaSubmissionResponseDto;
import nl.v2b.lalaland.presentation.dtos.SubmissionPendingDTO;
import nl.v2b.lalaland.security.CurrentUser;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.security.RolesAllowed;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/submission/image")
public class ImageSubmissionController {
	private final ImageSubmissionService service;

	public ImageSubmissionController(ImageSubmissionService service) {
		this.service = service;
	}

	@RolesAllowed("JUDGE")
	@GetMapping("/{challengeId}/all")
	public List<MediaSubmissionResponseDto> getAllImageSubmissionsByChallenge(@PathVariable int challengeId) {
		return service.getAllImageSubmissionsByChallengeId(challengeId).stream().map(MediaSubmissionResponseDto::new).collect(Collectors.toList());
	}
	@RolesAllowed("JUDGE")
	@GetMapping("/{challengeId}/pending")
	public List<SubmissionPendingDTO> getAllPendingImageSubmissionsByChallenge(@PathVariable int challengeId) {
		return service.getAllPendingSubmissionsByChallengeId(challengeId).stream().map(SubmissionPendingDTO::new).collect(Collectors.toList());
	}

	@GetMapping("/{challengeId}")
	public List<MediaSubmissionResponseDto> getAllImageSubmissionsByUserAndChallenge(@CurrentUser User user, @PathVariable int challengeId) {
		return service.getAllImageSubmissionsByUserAndChallenge(challengeId, user).stream().map(MediaSubmissionResponseDto::new).collect(Collectors.toList());
	}

	@GetMapping("/user")
	public List<MediaSubmissionResponseDto> getAllImageSubmissionsByUser(@CurrentUser User user){
		return service.getAllImageSubmissionsByUser(user).stream().map(MediaSubmissionResponseDto::new).collect(Collectors.toList());
	}

	@RolesAllowed("PLAYER")
	@PostMapping("/{challengeId}")
	public MediaSubmissionResponseDto doImageSubmission(@CurrentUser User user, @PathVariable int challengeId, @RequestBody SubmissionDTO submissionDTO) {
		ImageSubmission submission = service.submitImage(challengeId, user, submissionDTO.getAttempt());
		return new MediaSubmissionResponseDto(submission);
	}
	@Setter
	static class JudgeDTO {
		private boolean accepted;
	}
	@RolesAllowed("JUDGE")
	@PatchMapping("/{submissionId}/judge")
	public ResponseEntity<MediaSubmissionResponseDto> judgeSubmission(@PathVariable int submissionId, @RequestBody JudgeDTO judgeDTO) {
		try {
			return new ResponseEntity<>(new MediaSubmissionResponseDto(service.judgeSubmission(submissionId, judgeDTO.accepted)), HttpStatus.OK);
		} catch (Exception e) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
	}
    @RolesAllowed("JUDGE")
    @GetMapping("/all")
    public List<SubmissionPendingDTO> getAllSubmissions(){
        return service.getAllImageSubmissions().stream().map(SubmissionPendingDTO::new).collect(Collectors.toList());
    }

}
