package nl.v2b.lalaland.presentation;
import nl.v2b.lalaland.application.TextSubmissionService;
import nl.v2b.lalaland.domain.User;
import nl.v2b.lalaland.domain.submission.TextSubmission;
import nl.v2b.lalaland.presentation.dtos.SubmissionDTO;
import nl.v2b.lalaland.presentation.dtos.TextSubmissionDTO;
import nl.v2b.lalaland.security.CurrentUser;
import org.springframework.web.bind.annotation.*;


import jakarta.annotation.security.RolesAllowed;
import java.util.List;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/api/submission/text")
public class TextSubmissionController {
    private final TextSubmissionService service;

    public TextSubmissionController(TextSubmissionService service){
        this.service = service;
    }
    @RolesAllowed("JUDGE")
    @GetMapping("/{challengeId}/all")
    public List<TextSubmissionDTO> getAllTextSubmissionsByChallenge(@PathVariable int challengeId){
        return service.getAllTextSubmissionsByChallengeId(challengeId).stream().map(TextSubmissionDTO::new).collect(Collectors.toList());
    }

//  TODO: for some reason word dit aangeroepen bij het insturen van een text submission
    @GetMapping("/{challengeId}")
    @RolesAllowed("PLAYER")
    public List<TextSubmissionDTO> getAllTextSubmissionsByUser(@CurrentUser User user, @PathVariable int challengeId){
        return service.getAllTextSubmissionsByUserAndChallenge(challengeId, user).stream().map(TextSubmissionDTO::new).collect(Collectors.toList());
    }

    //todo block-attempt if user has already guessed it.
    @PostMapping("/{challengeId}")
    @RolesAllowed("PLAYER")
    public TextSubmissionDTO doTextSubmission(@CurrentUser User user, @PathVariable int challengeId, @RequestBody SubmissionDTO attempt) {
        TextSubmission submission = service.doTextSubmission(challengeId, user, attempt.getAttempt());
        return new TextSubmissionDTO(submission);
    }
}
