package nl.v2b.lalaland.zoom;

import nl.v2b.lalaland.domain.User;
import nl.v2b.lalaland.security.CurrentUser;
import nl.v2b.lalaland.zoom.dto.ZoomAllDTO;
import nl.v2b.lalaland.zoom.dto.ZoomCreateDTO;
import nl.v2b.lalaland.zoom.dto.ZoomJoinDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.security.RolesAllowed;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/zoom")
public class ZoomController {
    private ZoomMeetingService service;

    @Autowired
    public ZoomController(ZoomMeetingService service) {
        this.service = service;
    }
    @GetMapping("/all")
    public List<ZoomAllDTO> getAll() {
        return service.getAll().stream().map(ZoomAllDTO::new).collect(Collectors.toList());
    }
    @RolesAllowed("ADMIN")
    @PostMapping("/create")
    public Zoom createMeeting(@RequestBody ZoomCreateDTO dto) {
        return service.createMeeting(dto);
    }
    @GetMapping("/join/{id}")
    public ZoomJoinDTO joinMeeting(@CurrentUser User user, @PathVariable int id) {
        return service.generateZoomJoinDTO(user,id);
    }
    @RolesAllowed("ADMIN")
    @DeleteMapping("/{id}")
    public void removeMeeting(@PathVariable int id) {
        service.delete(id);
    }
}
