package nl.v2b.lalaland.presentation;

import nl.v2b.lalaland.application.CategoryService;
import nl.v2b.lalaland.domain.challenges.Challenge;
import nl.v2b.lalaland.presentation.dtos.CategoryDTO;
import nl.v2b.lalaland.security.SecurityHelper;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.RequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import java.util.ArrayList;
import static org.hamcrest.Matchers.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
class CategoryControllerTest {
	@Autowired
	private MockMvc mockMvc;

	@Autowired
	private CategoryService service;

	@Autowired
	private SecurityHelper helper;

	private int id;
	private int fakeId;
	ArrayList<Challenge> challengeList;
	private RequestBuilder category;
	private RequestBuilder fakeCategory;

	@BeforeEach
	void construct() {
		this.fakeId = 1000000000;
		this.challengeList = new ArrayList<>();

		CategoryDTO categoryDTO = new CategoryDTO("title");

		this.id = service.createCategory(categoryDTO).getId();

		this.category = MockMvcRequestBuilders
				.get("/api/category/" + this.id).with(helper.oidcAdmin())
				.contentType(MediaType.APPLICATION_JSON);

		this.fakeCategory = MockMvcRequestBuilders
				.get("/api/category/" + this.fakeId).with(helper.oidcAdmin());
	}

	@AfterEach()
	void decontsruct() {
		this.service.deleteCategory(this.id);
	}

	@Test
	@DisplayName("Get all users returns 200 response code.")
	void getCategoriesTest() throws Exception {
		RequestBuilder request = MockMvcRequestBuilders
				.get("/api/category/all").with(helper.oidcAdmin());
		mockMvc.perform(request)
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Get specific Category returns 200 response code.")
	void getCategorySuccesfullyTest() throws Exception {
		mockMvc.perform(this.category)
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id", notNullValue()))
				.andExpect(jsonPath("$.title", is("title")))
				.andExpect(jsonPath("$.challenges", hasSize(0)));
	}

	@Test
	@DisplayName("Get specific Category returns 404 response code.")
	void getCategoryFailedTest() throws Exception {
		mockMvc.perform(this.fakeCategory)
				.andExpect(status().isNotFound());
	}

	@Test
	@DisplayName("Delete specific Category.")
	void deleteCategoryTest() throws Exception {

		int id2 = service.createCategory(new CategoryDTO("DeleteTitle")).getId();

		mockMvc.perform(MockMvcRequestBuilders
				.delete("/api/category/delete/" + id2).with(helper.oidcAdmin())
				.contentType(MediaType.APPLICATION_JSON));

		mockMvc.perform(MockMvcRequestBuilders
				.get("/api/category/" + id2).with(helper.oidcAdmin())
				.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isNotFound());
	}

	@Test
	@DisplayName("update a Category with a code of 200.")
	void updateCategoryTest() throws Exception {
		mockMvc.perform(MockMvcRequestBuilders
				.patch("/api/category/update/"+this.id).with(helper.oidcAdmin())
				.contentType(MediaType.APPLICATION_JSON)
				.content("{\"id\":1,\"title\":\"CategoryTest\",\"challenges\":[]}").with(helper.oidcAdmin()));

		mockMvc.perform(this.category)
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id", notNullValue()))
				.andExpect(jsonPath("$.title", is("CategoryTest")))
				.andExpect(jsonPath("$.challenges", hasSize(0)));
	}
}
