package nl.v2b.lalaland.security;

import nl.v2b.lalaland.data.SpringUserRepository;
import nl.v2b.lalaland.domain.Admin;
import nl.v2b.lalaland.domain.Judge;
import nl.v2b.lalaland.domain.Player;
import nl.v2b.lalaland.domain.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors;
import org.springframework.stereotype.Component;

import java.util.UUID;
import java.util.stream.Collectors;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.oidcLogin;

@Component
public class SecurityHelper {
	@Autowired
	private SpringUserRepository repository;

	public SecurityMockMvcRequestPostProcessors.OidcLoginRequestPostProcessor oidcPlayer() {
		return security(new Player());
	}
	public SecurityMockMvcRequestPostProcessors.OidcLoginRequestPostProcessor oidcAdmin() {
		return security(new Admin());
	}
	public SecurityMockMvcRequestPostProcessors.OidcLoginRequestPostProcessor oidcJudge() {
		return security(new Judge());
	}
	private SecurityMockMvcRequestPostProcessors.OidcLoginRequestPostProcessor security(User user) {
		String id=UUID.randomUUID().toString();
		user.setAuth0(id);
		repository.save(user);
		return oidcLogin()
				.idToken(token -> token.claim("sub", id))
				.authorities(user.getRoles().stream().map(item->new SimpleGrantedAuthority("ROLE_"+item)).collect(Collectors.toList()));
	}
}
