package nl.v2b.lalaland.security;

import nl.v2b.lalaland.data.SpringUserRepository;
import nl.v2b.lalaland.domain.Player;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.web.servlet.MockMvc;

import java.security.SecureRandom;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
@SpringBootTest
@AutoConfigureMockMvc
class SecurityTest {
	@Autowired
	private MockMvc mockMvc;
	@Autowired
	private SpringUserRepository repository;
	@Autowired
	private SecurityHelper helper;
	private Player player;
	@BeforeEach
	void init() {
		player=new Player();
		player.setAuth0(Integer.toString(new SecureRandom().nextInt()));
		player=repository.save(player);
	}
	@Test
	void testPlayerAllowed() throws Exception {
		mockMvc.perform(get("/api/user/this").with(helper.oidcPlayer())).andExpect(status().isOk());
	}
	@Test
	void testAdminOnlyAsPlayer() throws Exception {
		mockMvc.perform(get("/api/user/"+player.getId()).with(helper.oidcPlayer())).andExpect(status().is(403));
	}
	@Test
	void testAdminOnlyAsAdmin() throws Exception {
		mockMvc.perform(get("/api/user/"+player.getId()).with(helper.oidcAdmin())).andExpect(status().isOk());
	}
	@Test
	void testAdminOnlyAsJudge() throws Exception {
		mockMvc.perform(get("/api/user/"+player.getId()).with(helper.oidcJudge())).andExpect(status().is(403));
	}
}
