import Vuex from "vuex";
import VuexPersistence from "vuex-persist";
import http from "@/http-common";
const vuexLocalStorage = new VuexPersistence({
  key: "vuex",
  storage: window.localStorage,
});
const initialState = {
  fullname: null,
  id: null,
  email: null,
  score: null,
  blocked: null,
  paid: null,
  picture: null,
  userType: null,
};
const store = new Vuex.Store({
  state: initialState,
  getters: {
    loggedIn: (state) => !!state.fullname,
    isAdmin: (state) => state.userType === "Admin",
    isJudge: (state) => state.userType === "Judge",
    isPlayer: (state) => state.userType === "Player",
  },
  mutations: {
    login: (state, newState) => {
      Object.assign(state, newState);
    },
    logout: (state) => {
      Object.assign(state, initialState);
    },
  },
  actions: {
    login(context) {
      return http
        .get("/user/this")
        .then((resp) => {
          context.commit("login", resp.data);
          return resp;
        })
        .catch((err) => {
          console.log(err);
          context.commit("logout");
        });
    },
    logout(context) {
      context.commit("logout");
    },
  },
  plugins: [vuexLocalStorage.plugin],
});
export default store;
// state: {},
// mutations: {},
// actions: {},
// modules: {},
